<?php
/**
 * Plugin Name: Crypto.com Pay Checkout for WooCommerce
 * Plugin URI:  http://www.crypto.com/
 * Description: Accept cryptocurrency using Crypto.com Pay Checkout.
 * Author:      Crypto.com
 * Author URI:  mailto:tech@crypto.com?subject=Crypto.com Pay Checkout for WooCommerce
 * Version:     1.1.0
 *
 * @package     Crypto/Classes
 */

/**
 * Copyright (c) 2019 Crypto.com
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

define('CRYPTO_PLUGIN_VERSION', '1.0.0');

/**
 * add or update plugin version to database
 */
function cp_crypto_save_plugin_version()
{
    $crypto_plugin_version = get_option('crypto_plugin_version');
    if (!$crypto_plugin_version) {
        add_option('crypto_plugin_version', CRYPTO_PLUGIN_VERSION);
    } else {
        update_option('crypto_plugin_version', CRYPTO_PLUGIN_VERSION);
    }
}

register_activation_hook(__FILE__, 'cp_crypto_save_plugin_version');

add_action('plugins_loaded', 'cp_load_crypto_payment_gateway', 0);

ob_start();

/**
 * notice message when WooCommerce is not active
 */
function cp_notice_to_activate_woocommerce()
{

    echo '<div id="message" class="error notice is-dismissible"><p><strong>Crypto.com Pay Checkout: </strong>' .
    esc_attr(__('WooCommerce must be active to make this plugin working properly', 'crypto-pay')) .
        '</p></div>';
}

/**
 * Init payment gateway
 */
function cp_load_crypto_payment_gateway()
{

    /**
     * Loads translation
     */
    load_plugin_textdomain('crypto-pay', false, dirname(plugin_basename(__FILE__)) . '/languages/');

    if (!class_exists('WC_Payment_Gateway')) {
        add_action('admin_notices', 'cp_notice_to_activate_woocommerce');
        return;
    }

    include_once dirname(__FILE__) . '/includes/class-crypto-helper.php';
    include_once dirname(__FILE__) . '/includes/class-crypto-payment-api.php';

    if (!class_exists('Crypto_Pay')) {

        /**
         * Crypto Payment Gateway
         *
         * @class Crypto_Pay
         */
        class Crypto_Pay extends WC_Payment_Gateway
        {

            public $id = 'crypto_pay';

            /**
             * Woocommerce order
             *
             * @var object $wc_order
             */
            protected $wc_order;

            /**
             * Main function
             */
            public function __construct()
            {
                $plugin_dir = plugin_dir_url(__FILE__);
                $this->form_fields = $this->get_crypto_form_fields();
                $this->method_title = __('Crypto.com', 'crypto-pay');
                $this->method_description = __('Accept Bitcoin and more cryptocurrencies without the risk of price fluctuation.', 'crypto-pay');
                $this->icon = apply_filters('woocommerce_gateway_icon', '' . $plugin_dir . '/assets/icon.svg');

                $this->supports = array('products', 'refunds');

                $this->init_settings();

                // action to save crypto pay backend configuration
                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
                // action to show payment page
                add_action('woocommerce_receipt_' . $this->id, array(&$this, 'payment_state'));
                // action to show success page
                add_action('woocommerce_thankyou_' . $this->id, array(&$this, 'success_state'));

                if (isset(WC()->session->crypto_success_state)) {
                    unset(WC()->session->crypto_success_state);
                }
                if (isset(WC()->session->crypto_payment_state)) {
                    unset(WC()->session->crypto_payment_state);
                }
                if (isset(WC()->session->crypto_display_error)) {
                    $_POST['crypto_error'] = '1';
                    unset(WC()->session->crypto_display_error);
                }
            }

            /**
             * Get payment method title
             *
             * @return string
             */
            public function get_title()
            {
                return $this->method_title;
            }

            /**
             * set crypto backend configuration fields
             */
            public function get_crypto_form_fields()
            {

                $form_fields = array(
                    'enabled' => array(
                        'title' => __('Enabled', 'crypto-pay'),
                        'type' => 'checkbox',
                        'default' => '',
                    ),
                    'test_publishable_key' => array(
                        'title' => __('Test Publishable Key', 'crypto-pay'),
                        'type' => 'password',
                        'default' => '',
                    ),
                    'test_secret_key' => array(
                        'title' => __('Test Secret Key', 'crypto-pay'),
                        'type' => 'password',
                        'default' => '',
                    ),
                    'live_publishable_key' => array(
                        'title' => __('Live Publishable Key', 'crypto-pay'),
                        'type' => 'password',
                        'default' => '',
                    ),
                    'live_secret_key' => array(
                        'title' => __('Live Secret Key', 'crypto-pay'),
                        'type' => 'password',
                        'default' => '',
                    ),
                    'environment' => array(
                        'title' => __('Environment', 'crypto-pay'),
                        'type' => 'select',
                        'description' => __('Select <b>Test</b> for testing the plugin, <b>Production</b> when you are ready to go live.'),
                        'options' => array(
                            'production' => 'Production',
                            'test' => 'Test',
                        ),
                        'default' => 'test',
                    ),
                );

                return $form_fields;
            }

            /**
             * Process the payment
             *
             * @param int $order_id order id.
             * @return array
             */
            public function process_payment($order_id)
            {
                $order = new WC_Order($order_id);
                return array(
                    'result' => 'success',
                    'redirect' => $order->get_checkout_payment_url(true),
                );
            }

            /**
             * Calls from hook "woocommerce_receipt_{gateway_id}"
             *
             * @param int $order_id order id.
             */
            public function payment_state($order_id)
            {
                $payment_id = Crypto_Helper::get_request_value('id');
                $error_payment = Crypto_Helper::get_request_value('error');

                if (!empty($payment_id)) {
                    $this->crypto_process_approved_payment($order_id, $payment_id);
                } elseif (!empty($error_payment)) {
                    $this->crypto_process_error_payment($order_id, 'wc-failed', 'payment failed');
                }

                if (!isset(WC()->session->crypto_payment_state)) {
                    $this->crypto_render_payment_button($order_id);
                    WC()->session->set('crypto_payment_state', true);
                }
            }

            /**
             * render crypto payment button
             *
             * @param int $order_id order id.
             */
            private function crypto_render_payment_button($order_id)
            {
                global $wp;
                // if ( 'USD' !== get_woocommerce_currency() ) {
                //     $this->crypto_process_error_payment( $order_id, 'wc-failed', 'currency not allowed' );
                // }

                $payment_parameters = $this->get_crypto_payment_parameters($order_id);
                $key = Crypto_Helper::get_request_value('key');
                $order_pay = Crypto_Helper::get_request_value('order-pay');

                if (isset($wp->request)) {
                    $result_url = $this->crypto_get_home_url($wp->request) . 'key=' . $key;
                    $result_url = str_replace("order-pay", "order-received", $result_url);
                } else {
                    $result_url = get_page_link() . '&order-pay=' . $order_pay . '&key=' . $key;
                }

                $args = array(
                    'result_url' => $result_url,
                    'payment_parameters' => $payment_parameters,
                );

                $path = dirname(__FILE__) . '/templates/checkout/template-payment-button.php';
                Crypto_Helper::set_template($path, $args);
            }

            /**
             * Get base url
             *
             * @param string $wp_request wp request
             * @return string
             */
            private function crypto_get_home_url($wp_request)
            {
                if (false !== strpos(home_url($wp_request), '/?')) {
                    $home_url = home_url($wp_request) . '&';
                } else {
                    $home_url = home_url($wp_request) . '/?';
                }
                return $home_url;
            }

            /**
             * check payment status with payment id
             *
             * @param int $order_id order id.
             * @param string $payment_id payment id.
             */
            private function crypto_process_approved_payment($order_id, $payment_id)
            {

                // check payment status with payment_id
                // TODO: Review the usage of this function [Thomas, 20201027]

                $this->crypto_show_success_page($order_id);
            }

            /**
             * cancel the order
             *
             * @param int $order_id order id.
             */
            private function crypto_cancel_order($order_id)
            {
                $this->crypto_process_error_payment($order_id, 'wc-cancelled', 'cancelled by user');
            }

            /**
             * set order status, reduce stock, empty cart and show success page.
             *
             * @param int     $order_id order id.
             */
            private function crypto_show_success_page($order_id)
            {
                $order = new WC_Order($order_id);
                wc_reduce_stock_levels($order_id);
                WC()->cart->empty_cart();
                wp_safe_redirect($this->get_return_url($order));
                exit();
            }

            /**
             * Error payment action
             *
             * @param int          $order_id order id.
             * @param string       $payment_status payment status.
             * @param string|array $error_message error identifier.
             */
            private function crypto_process_error_payment($order_id, $payment_status, $error_message = 'payment error')
            {
                global $woocommerce;

                $order = new WC_Order($order_id);

                // Cancel the order.
                $order->update_status($error_message);
                $order->update_status($payment_status, 'order_note');

                // To display failure messages from woocommerce session.
                if (isset($error_message)) {
                    $woocommerce->session->errors = $error_message;
                    wc_add_notice($error_message, 'error');
                    WC()->session->set('crypto_display_error', true);
                }

                wp_safe_redirect(wc_get_checkout_url());
                exit();
            }

            /**
             * Calls from hook "woocommerce_thankyou_{gateway_id}"
             */
            public function success_state($order_id)
            {

                // 1.1.0 update: Update metadata here so we can process refund from woocommerce
                $payment_id = Crypto_Helper::get_request_value('id');
                $order = wc_get_order($order_id);
                $order->add_meta_data('crypto_pay_paymentId', $payment_id);
                $order->save_meta_data();

                if (!isset(WC()->session->crypto_success_state)) {
                    WC()->session->set('crypto_success_state', true);
                }
            }

            /**
             * get customer parameters by order
             *
             * @return array
             */
            private function crypto_get_customer_parameters()
            {
                $customer['first_name'] = $this->wc_order->get_billing_first_name();
                $customer['last_name'] = $this->wc_order->get_billing_last_name();
                $customer['email'] = $this->wc_order->get_billing_email();
                $customer['phone'] = $this->wc_order->get_billing_phone();

                return $customer;
            }

            /**
             * get billing parameters by order
             *
             * @return array
             */
            private function crypto_get_billing_parameters()
            {
                $billing['address'] = $this->wc_order->get_billing_address_1();
                $billing_address_2 = trim($this->wc_order->get_billing_address_2());
                if (!empty($billing_address_2)) {
                    $billing['address'] .= ', ' . $billing_address_2;
                }
                $billing['city'] = $this->wc_order->get_billing_city();
                $billing['postcode'] = $this->wc_order->get_billing_postcode();
                $billing['country'] = $this->wc_order->get_billing_country();

                return $billing;
            }

            /**
             * get payment parameters by order
             *
             * @param int $order_id order id.
             * @return array
             */
            private function get_crypto_payment_parameters($order_id)
            {
                $this->wc_order = new WC_Order($order_id);

                $payment_parameters['publishable_key'] = ($this->settings['environment'] == 'production' ? $this->settings['live_publishable_key'] : $this->settings['test_publishable_key']);
                $payment_parameters['order_id'] = $order_id;
                $payment_parameters['amount'] = (float) $this->get_order_total() * 100;
                $payment_parameters['currency'] = get_woocommerce_currency();
                $payment_parameters['customer'] = $this->crypto_get_customer_parameters();
                $payment_parameters['billing'] = $this->crypto_get_billing_parameters();
                $payment_parameters['description'] = "WooCommerce order ID: $order_id";
                $payment_parameters['first_name'] = $this->wc_order->get_billing_first_name();
                $payment_parameters['last_name'] = $this->wc_order->get_billing_last_name();

                return $payment_parameters;
            }

            /**
             * Process refund. // 1.1.0 update
             *
             * @param int    $order_id Order ID
             * @param float  $amount   Order amount
             * @param string $reason   Refund reason
             *
             * @return boolean True or false based on success, or a WP_Error object.
             */
            public function process_refund($order_id, $amount = null, $reason = '')
            {
                $order = wc_get_order($order_id);

                if (0 == $amount || null == $amount) { // phpcs:ignore WordPress.PHP.StrictComparisons.LooseComparison
                    return new WP_Error('crypto_pay_refund_error', __('Refund Error: You need to specify a refund amount.', 'crypto-pay'));
                }

                // actually woo converts to 2 d.p. automatically so unable to catch this, but just in case
                if ($this->get_decimal_count($amount) > 2) {
                    return new WP_Error('crypto_pay_refund_error', __('Refund Error: Refund amount cannot be larger than 2 decimal places.', 'crypto-pay'));
                }

                $secret_key = ($this->settings['environment'] == 'production' ? $this->settings['live_secret_key'] : $this->settings['test_secret_key']);
                $payment_id = $order->get_meta('crypto_pay_paymentId', true);
                $order_currency = $order->get_currency();

                if (!isset($payment_id)) {
                    return new WP_Error('crypto_pay_refund_error', __('Refund Error: This order cannot be refunded automatically as custom field `crypto_pay_paymentId` is not present.', 'crypto-pay'));
                }

                $result = Crypto_Payment_Api::request_refund($payment_id, $order_id, $order_currency, $amount, $reason, $secret_key);

                if (isset($result['error'])) {
                    return new WP_Error('crypto_pay_refund_error', __('Refund Error: ' . ($result['error']['message'] ?? print_r($result, true)), 'crypto-pay'));
                }

                return true;
            }

            /**
             * get number of decimals from a number // 1.1.0 update
             *
             * @param f number to evaluate
             * @return int number of decimals
             */
            private function get_decimal_count($f)
            {
                $num = 0;
                while (true) {
                    if ((string) $f === (string) round($f)) {
                        break;
                    }
                    if (is_infinite($f)) {
                        break;
                    }

                    $f *= 10;
                    $num++;
                }
                return $num;
            }
        }
    }

    /**
     * Add Crypto Pay to WooCommerce
     *
     * @access public
     * @param array $gateways gateways.
     * @return array
     */
    function crypto_add_to_gateways($gateways)
    {
        $gateways[] = 'Crypto_Pay';
        return $gateways;
    }
    add_filter('woocommerce_payment_gateways', 'crypto_add_to_gateways');
}
